; ---------------------------------------------------------
;	H8/3048F LCD(4bit interface) access test
; =========================================================
;	(C)2004 t.hara
; ---------------------------------------------------------

	.cpu	300HA

; ---------------------------------------------------------
;	CPU registers
ABWCR		.equ		H'FFFFEC			; area bus width control register
BRCR		.equ		H'FFFFF3			; bus release controller enable register

; ---------------------------------------------------------
;	for SRAM board				(portA)
LcdDir		.equ		H'FFFFD1			; LCD port data direction register
LcdData		.equ		H'FFFFD3			; LCD port data register
;	for AKI-3048F micom board	(port3)
;LcdDir		.equ		H'FFFFC4			; LCD port data direction register
;LcdData	.equ		H'FFFFC6			; LCD port data register

LcdEN		.bequ		5,LcdData			; LCD signal E
LcdRS		.bequ		4,LcdData			; LCD signal RS

; ---------------------------------------------------------
;	reset vector
			.section	reset0, data, locate=H'00000
			.data.l		ResetEntry

; ---------------------------------------------------------
			.section	rom, code, locate=H'00100
ResetEntry:
			mov.l		#H'FFFF10, ER7		; set up StackPointer

; ---------------------------------------------------------
;	port initialize
			mov.b		#H'FF, R0L			; set output mode to LCD port
			mov.b		R0L, @LcdDir

; ---------------------------------------------------------
;	LCD initialize
			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100010, R0L	; E=1, RS=0, D=02H interface 4bit data length
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms
			mov.b		#B'00101000, R0L	; function set, DL=4bit, F=5x7dot matrix
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000110, R0L	; entry mode set, I/D=increment, S=shift off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001000, R0L	; display on/off, D=display off, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000001, R0L	; display clear
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001100, R0L	; display on/off, D=display on, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms

; ---------------------------------------------------------
;	LCD Test ok
			mov.l		#LcdTest, ER1		; display a message
			jsr			@LcdPuts

; ---------------------------------------------------------
;	End of test
EndOfTest:
			sleep							; low power
			bra			EndOfTest

; ---------------------------------------------------------
;	Wait 4 millisecond
Wait4ms:
			push.l		ER0
			mov.l		#H'2000, ER0
Wait4msLoop:
			sub.l		#1, ER0
			bne			Wait4msLoop
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Wait LCD
WaitLcd:
			push.l		ER0
			mov.l		#170, ER0
WaitLcdLoop:
			sub.l		#1, ER0
			bne			Wait4msLoop
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send command to LCD controller
;	input:
;		R0L ... send command byte
;
LcdCommand:
			push.l		ER0
;	high 4bit
			bclr		LcdRS				; RS=0
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1(RSύX 40ns ȏ), RS=0
			or.b		#B'00100000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd			; 230ns ȏ
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			bclr		LcdRS				; RS=0
			and.b		#B'00001111, R0H	; E=1(RSύX 40ns ȏ), RS=0
			or.b		#B'00100000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd			; 230ns ȏ
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send character to LCD controller
;	input:
;		R0L ... send character byte
;
LcdPutc:
			push.l		ER0
;	high 4bit
			bclr		LcdRS				; RS=1
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1(RSύX 500ns ȏ), RS=1
			or.b		#B'00110000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			bclr		LcdRS				; RS=1
			and.b		#B'00001111, R0H	; E=1(RSύX 500ns ȏ), RS=1
			or.b		#B'00110000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send string to LCD controller
;	input:
;		ER1 ... address of string
LcdPuts:
			push.l		ER0
			push.l		ER1
LcdPutsLoop:
			mov.b		@ER1, R0L
			beq			LcdPutsLoopExit
			jsr			@LcdPutc
			add.l		#1, ER1
			bra			LcdPutsLoop
LcdPutsLoopExit:
			pop.l		ER1
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	datas
LcdTest		.sdata		"LCD Test ... Ok"
			.data.b		0

			.end
