; ---------------------------------------------------------
;	H8/3048F SRAM access test
; =========================================================
;	(C)2004 t.hara
; ---------------------------------------------------------

	.cpu	300HA

; ---------------------------------------------------------
;	CPU registers
ABWCR		.equ		H'FFFFEC			; area bus width control register
BRCR		.equ		H'FFFFF3			; bus release controller enable register

; ---------------------------------------------------------
LcdDir		.equ		H'FFFFD1			; LCD port data direction register
LcdData		.equ		H'FFFFD3			; LCD port data register

LcdEN		.bequ		5,LcdData			; LCD signal E
LcdRS		.bequ		4,LcdData			; LCD signal RS

; ---------------------------------------------------------
;	reset vector
			.section	reset0, data, locate=H'00000
			.data.l		ResetEntry

; ---------------------------------------------------------
			.section	rom, code, locate=H'00100
ResetEntry:
			mov.l		#H'FFFF10, ER7		; set up StackPointer

; ---------------------------------------------------------
;	port initialize
			mov.b		#H'FF, R0L			; set output mode to LCD port
			mov.b		R0L, @LcdDir

			mov.b		#H'FF, R0L			; set output mode to Address out port
			mov.b		R0L, @H'FFFFC0		; 	port1 (A7-A0)
			mov.b		R0L, @H'FFFFC1		;	port2 (A15-A8)
			mov.b		R0L, @H'FFFFC8		;	port5 (A19-A16)
			mov.b		#H'E8, R0L
			mov.b		R0L, @H'FFFFCD		;	port8 (/CS1)

; ---------------------------------------------------------
;	LCD initialize
			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100010, R0L	; E=1, RS=0, D=02H interface 4bit data length
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms
			mov.b		#B'00101000, R0L	; function set, DL=4bit, F=5x7dot matrix
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000110, R0L	; entry mode set, I/D=increment, S=shift off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001000, R0L	; display on/off, D=display off, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000001, R0L	; display clear
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001100, R0L	; display on/off, D=display on, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms

; ---------------------------------------------------------
;	LCD Test ok
			mov.l		#LcdTest, ER1		; display a message
			jsr			@LcdPuts

; ---------------------------------------------------------
;	Check SRAM access

; ---------------------------------------------------------
;	Data bus
			mov.b		#H'C0, R0L			; locate( 0, 1 )
			jsr			@LcdCommand
			mov.l		#SramDTest, ER1		; display a message
			jsr			@LcdPuts

			mov.l		#H'200000, ER2		; top of SRAM address
			mov.b		#0, R5L
ByteCheckLoop2:
			mov.b		#0, R4L
ByteCheckLoop:
			mov.b		R4L, @ER2			; write test
			mov.b		@ER2, R4H			; read test
			cmp.b		R4L, R4H
			bne			SramError
			add.b		#1, R4L
			bne			ByteCheckLoop
			dec			R5L
			bne			ByteCheckLoop2

; ---------------------------------------------------------
;	Address bus
			mov.b		#H'C0, R0L			; locate( 0, 1 )
			jsr			@LcdCommand
			mov.l		#SramATest, ER1		; display a message
			jsr			@LcdPuts

			mov.l		#H'200000, ER5		; A0
			mov.l		#H'200001, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A1
			mov.l		#H'200002, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A2
			mov.l		#H'200004, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A3
			mov.l		#H'200008, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A4
			mov.l		#H'200010, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A5
			mov.l		#H'200020, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A6
			mov.l		#H'200040, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A7
			mov.l		#H'200080, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A8
			mov.l		#H'200100, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A9
			mov.l		#H'200200, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A10
			mov.l		#H'200400, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A11
			mov.l		#H'200800, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A12
			mov.l		#H'201000, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A13
			mov.l		#H'202000, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A14
			mov.l		#H'204000, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A15
			mov.l		#H'208000, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A16
			mov.l		#H'210000, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A17
			mov.l		#H'220000, ER6
			jsr			@SramATestSub

			mov.l		#H'200000, ER5		; A18
			mov.l		#H'240000, ER6
			jsr			@SramATestSub

; ---------------------------------------------------------
;	Memory test
			mov.b		#H'C0, R0L			; locate( 0, 1 )
			jsr			@LcdCommand
			mov.l		#SramMTest, ER1		; display a message
			jsr			@LcdPuts

			mov.l		#H'200000, ER2
			mov.l		#H'80000, ER3
MemCheckLoop2:
			mov.b		#0, R4L
MemCheckLoop:
			mov.b		R4L, @ER2			; write test
			mov.b		@ER2, R4H			; read test
			cmp.b		R4L, R4H
			bne			SramError
			add.b		#1, R4L
			bne			MemCheckLoop

			mov.b		#H'94, R0L			; locate( 0, 2 )
			jsr			@LcdCommand
			add.l		#1, ER2
			mov.l		ER2, ER1
			jsr			@DumpHex
			sub.l		#1, ER3
			bne			MemCheckLoop2

			bra			ExitSramTest

			mov.l		#TestOk, ER1
			jsr			@LcdPuts

; ---------------------------------------------------------
SramError:
			mov.b		#H'94, R0L			; locate( 0, 2 )
			jsr			@LcdCommand
			mov.l		ER6, ER1			; display address
			jsr			@DumpHex

			mov.b		#H'D4, R0L			; locate( 0, 3 )
			jsr			@LcdCommand
			mov.l		#0, ER1
			mov.b		R4L, R1L
			jsr			@DumpHex			; display write data

			mov.b		#32, R0L			; display ' '
			jsr			@LcdPutc

			mov.l		#0, ER1
			mov.b		R4H, R1L
			jsr			@DumpHex			; display read data

			mov.l		#TestNg, ER1
			jsr			@LcdPuts
ExitSramTest:
; ---------------------------------------------------------
;	End of test
EndOfTest:
			sleep							; low power
			bra			EndOfTest

; ---------------------------------------------------------
SramATestSub:
			mov.b		#H'AA, R1L
			mov.b		#H'55, R1H
			mov.b		R1L, @ER5
			mov.b		R1H, @ER6
			mov.b		@ER5, R4L
			mov.b		@ER6, R4H
			cmp.b		R4L, R1L
			bne			SramError
			cmp.b		R4H, R1H
			bne			SramError
			rts

; ---------------------------------------------------------
;	Wait 4 millisecond
Wait4ms:
			push.l		ER0
			mov.l		#H'2000, ER0
Wait4msLoop:
			sub.l		#1, ER0
			bne			Wait4msLoop
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Wait LCD
WaitLcd:
			push.l		ER0
			mov.l		#170, ER0
WaitLcdLoop:
			sub.l		#1, ER0
			bne			Wait4msLoop
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send command to LCD controller
;	input:
;		R0L ... send command byte
;
LcdCommand:
			push.l		ER0
;	high 4bit
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1, RS=0
			or.b		#B'00100000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			and.b		#B'00001111, R0H	; E=1, RS=0
			or.b		#B'00100000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send character to LCD controller
;	input:
;		R0L ... send character byte
;
LcdPutc:
			push.l		ER0
;	high 4bit
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1, RS=1
			or.b		#B'00110000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			and.b		#B'00001111, R0H	; E=1, RS=1
			or.b		#B'00110000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Send string to LCD controller
;	input:
;		ER1 ... address of string
LcdPuts:
			push.l		ER0
			push.l		ER1
LcdPutsLoop:
			mov.b		@ER1, R0L
			beq			LcdPutsLoopExit
			jsr			@LcdPutc
			add.l		#1, ER1
			bra			LcdPutsLoop
LcdPutsLoopExit:
			pop.l		ER1
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	Hex code dump
;	input:
;		ER1 ... hex code
DumpHex:
			push.l		ER0
			push.l		ER1
			mov.b		#8, R0H
DumpHexLoop:
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			push.l		ER1
			and.l		#H'0000000F, ER1
			add.l		#HexChar, ER1
			mov.b		@ER1, R0L
			jsr			@LcdPutc
			pop.l		ER1
			dec			R0H
			bne			DumpHexLoop
			pop.l		ER1
			pop.l		ER0
			rts

; ---------------------------------------------------------
;	datas
LcdTest		.sdata		"LCD Test ... Ok"
			.data.b		0
SramDTest	.sdata		"SRAM D-BUS Test"
			.data.b		0
SramATest	.sdata		"SRAM A-BUS Test"
			.data.b		0
SramMTest	.sdata		"SRAM Memory Test"
			.data.b		0
TestOk		.sdata		"Ok"
			.data.b		0
TestNg		.sdata		"NG"
			.data.b		0
HexChar		.sdata		"0123456789ABCDEF"

			.end
